#!/bin/bash
#
# author           : Pedro Pinto (pnboy~AT~pinguix.com)
# latest update    : 09-03-2007
# source location  : http://www.webmin.com/
# url              : http://prdownloads.sourceforge.net/webadmin/webmin-1.340.tar.gz

NAME=webmin
VERSION=${VERSION:-1.400}
ARCH=${ARCH:-noarch}
BUILD=${BUILD:-52.1}
TMP=${TMP:-/tmp}
PKG=$TMP/package-$NAME
CWD=`pwd`
EXTENSION=tar.gz

#
# building-computer dependent: the number of parallel 'jobs' of 'make'
#
# 'heavy' computer: 6 build-jobs at once
#
numjobs='-j6'
#
# 'light' computer: 2 build-jobs at once
#
#numjobs='-j2'

rm -rf $PKG
mkdir -p $PKG

cd $TMP
rm -rf $NAME-$VERSION
  if [ -f $NAME-$VERSION.$EXTENSION ]; then
          echo "Source file already here, no need to re-download";
  elif [ -f $CWD/$NAME-$VERSION.$EXTENSION ]; then
          echo -n "Relocating source file to $TMP..."
          cp $CWD/$NAME-$VERSION.$EXTENSION $TMP/
	  echo "done"
  else
	  wget -c http://switch.dl.sourceforge.net/sourceforge/webadmin/$NAME-$VERSION.$EXTENSION
  fi
  tar xf $NAME-$VERSION.$EXTENSION

cd $NAME-$VERSION
rm -f mount/freebsd-mounts*
rm -f mount/openbsd-mounts*
rm -f mount/macos-mounts*
(find . -name '*.cgi' ; find . -name '*.pl') | perl perlpath.pl /usr/bin/perl -

mkdir -p $PKG/usr/libexec/webmin
mkdir -p $PKG/var/log/webmin
mkdir -p $PKG/etc/rc.d
mkdir -p $PKG/etc/webmin

cp -rp * $PKG/usr/libexec/webmin

cp webmin-init $PKG/etc/rc.d/rc.webmin
chmod +x $PKG/etc/rc.d/rc.webmin

echo "slackware">$PKG/usr/libexec/webmin/install-type

config_dir=$PKG/etc/webmin
var_dir=$PKG/var/log/webmin
perl=/usr/bin/perl
autoos=1
port=10000
login=root
crypt="XXX"
host=`hostname`
ssl=1
atboot=0
nostart=1
nochown=1
autothird=1
nouninstall=1
noperlpath=1
export config_dir var_dir perl autoos port login crypt host ssl atboot nostart nochown autothird nouninstall noperlpath

$PKG/usr/libexec/webmin/setup.sh

# Fix config files to use their real locations
sed -i -e 's/^pidfile=.*$/pidfile=\/var\/run\/webmin.pid/' $PKG/etc/webmin/miniserv.conf
find ${PKG}/etc/webmin -type f | xargs sed -i -e "s|$PKG||g"
mv $PKG/etc/webmin/miniserv.users $PKG/etc/webmin/miniserv.users.new
mv $PKG/etc/webmin/miniserv.conf $PKG/etc/webmin/miniserv.conf.new
mv $PKG/etc/rc.d/rc.webmin $PKG/etc/rc.d/rc.webmin.new

# Strip binaries:
( cd $PKG
  find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
  find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
)

# Compress the man page(s)
if [ -d $PKG/usr/man ]; then
	find $PKG/usr/man -type f -name "*.?" -exec gzip -9f {} \;
fi
if [ -d $PKG/usr/info ]; then
	find $PKG/usr/info -type f -name "*.info*" -exec gzip -9f {} \;
fi

# Add a documentation directory:
mkdir -p $PKG/usr/doc/$NAME-$VERSION

# The Makefile evidently contains the package's license...
DOCS="LICENCE* README"
cd $TMP/$NAME-$VERSION
cp -a \
  $DOCS \
  $PKG/usr/doc/$NAME-$VERSION

# 
# as per package rules: add the build-file itself
#
mkdir -p $PKG/usr/src/$NAME-$VERSION
cp "$CWD/build-$NAME.sh" "$PKG/usr/src/$NAME-$VERSION/build-$NAME.sh"
chmod a+x "$PKG/usr/src/$NAME-$VERSION/build-$NAME.sh"

mkdir -p $PKG/install

#    |-----handy-ruler------------------------------------------------------|
echo "$NAME: $NAME (Web-based administration interface) 
$NAME:
$NAME: Webmin is a web-based interface for system administration for Unix.
$NAME: Using any browser that supports tables and forms (and Java for the
$NAME: File Manager module), you can setup user accounts, Apache, DNS, file
$NAME: sharing and so on.
$NAME: Webmin consists of a simple web server, and a number of CGI programs
$NAME: which directly update system files like /etc/inetd.conf and
$NAME: /etc/passwd. The web server and all CGI programs are written in Perl
$NAME: version 5, and use no non-standard Perl modules.
$NAME: "> $PKG/install/slack-desc

cat <<"EOF" > $PKG/install/doinst.sh
config() {
  NEW="$1.new"
  OLD="$1"
  # If there's no config file by that name, mv it over:
  if [ ! -r $OLD ]; then
    mv $NEW $OLD
  elif [ "`cat $OLD | md5sum`" = "`cat $NEW | md5sum`" ]; then # toss the redundant copy
    rm $NEW
  fi
  # Otherwise, we leave the .new copy for netpkg to consider...
}
echo root:`cat etc/shadow | grep root | cut -f 2 -d :`:0 > etc/webmin/miniserv.users.new

config etc/webmin/miniserv.users
config etc/webmin/miniserv.conf
config etc/rc.d/rc.webmin
EOF
chmod +x $PKG/install/doinst.sh
#
# real packaging work:
#
# Make sure ownerships and permissions are sane:
cd $PKG
chown -R root:root .
find . -perm 666 -exec chmod 644 {} \;
find . -perm 664 -exec chmod 644 {} \;
find . -perm 600 -exec chmod 644 {} \;
find . -perm 444 -exec chmod 644 {} \;
find . -perm 400 -exec chmod 644 {} \;
find . -perm 440 -exec chmod 644 {} \;
find . -perm 777 -exec chmod 755 {} \;
find . -perm 775 -exec chmod 755 {} \;
find . -perm 511 -exec chmod 755 {} \;
find . -perm 711 -exec chmod 755 {} \;
find . -perm 555 -exec chmod 755 {} \;

makepkg -l y -c n ../$NAME-$VERSION-$ARCH-$BUILD.tgz

#
# computes the md5sum signature for the package
#
cd $TMP
md5sum $NAME-$VERSION-$ARCH-$BUILD.tgz > $NAME-$VERSION-$ARCH-$BUILD.md5
echo "perl" > $NAME-$VERSION-$ARCH-$BUILD.dep

